package com.anagog.anagog_flutter.app_initializer

import android.content.Context
import android.util.Log
import androidx.startup.Initializer
import com.anagog.anagog_flutter.AnagogFlutterPlugin
import com.anagog.anagog_flutter.jedai_sdk.JemaWrapper
import com.anagog.anagog_flutter.model.PluginConfig
import com.anagog.anagog_flutter.plugin_utils.PluginUtils
import com.anagog.jedai.core.logger.JedAILogger
import com.google.gson.Gson

private const val TAG = "PluginInitializer"
class FlutterPluginInitializer : Initializer<JemaWrapper> {
    override fun create(context: Context) : JemaWrapper {
        Log.i(TAG, "create: ")
        JemaWrapper.setup(context, readConfigFile(context))
        return JemaWrapper
    }

    override fun dependencies(): List<Class<out Initializer<*>>> {
        return emptyList()
    }

    private fun readConfigFile(context: Context): PluginConfig {
        return try {
            val configJson = PluginUtils.readJsonFile(context.assets, fileName = "anagog_config")
            val pluginConfig = Gson().fromJson(configJson, PluginConfig::class.java)
            Log.i(TAG, "readConfigFile: $pluginConfig")
            pluginConfig
        } catch (e: Throwable) {
            Log.e(TAG, "readConfigFile: Something went wrong: ${e.message}, Did you forgot to add anagog_config.json?\\nMake sure you have anagog_config.json in your android project root assets: assets/anagog_config.json")
            JemaWrapper.sLogger.error { "readConfigFile: Something went wrong: ${e.message}, Did you forgot to add anagog_config.json?\nMake sure you have anagog_config.json in your android project root assets: assets/anagog_config.json" }
            PluginConfig()
        }
    }
}