package com.anagog.anagog_flutter.jedai_sdk

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.graphics.BitmapFactory
import android.graphics.Color
import android.os.Build
import android.os.Bundle
import android.util.Log
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import com.anagog.anagog_flutter.R
import com.anagog.anagog_flutter.model.PluginConfig
import com.anagog.anagog_flutter.plugin_utils.ANAGOG_NOTIFICATION_ICON
import com.anagog.anagog_flutter.plugin_utils.getAppName

object NotificationHelper {

    fun setupForegroundNotification(context: Context, config: PluginConfig, appMetaData: Bundle): Notification {

        val channelId = config.foregroundChannelId?.let {
            if(it == config.notificationChannelId) { //Ensure notification channel not the same
                "${it}_FG"
            } else {
                it
            }
        } ?: "${context.getAppName()}_FG"
        createNotificationChannelsIfNeeded(
            context,
            channelId
        )

        val title = context.getString(R.string.anagog_foreground_title)
        val message = context.getString(R.string.anagog_foreground_message)

        Log.i("NotificationHelper", "setupForegroundNotification: Title: $title, Message: $message")

        val icon = try{
            appMetaData.getInt(ANAGOG_NOTIFICATION_ICON)
        } catch (e: Throwable){
            context.applicationInfo.icon
        }

        Log.i("NotificationHelper", "setupForegroundNotification: title: $title, message: $message")
        val builderCompat = NotificationCompat.Builder(context, channelId)
            .setContentTitle(title)
            .setContentText(message)
            .setContentIntent(null)
            .setSmallIcon(icon)
            .setPriority(NotificationCompat.PRIORITY_MIN)
            .setLargeIcon(
                BitmapFactory
                    .decodeResource(context.resources, icon)
            )
        return builderCompat.build()
    }

    /**
     * Create the NotificationChannel, but only on API 26+ because
     * the NotificationChannel class is new and not in the support library
     */
    private fun createNotificationChannelsIfNeeded(context: Context, channelName: String) {

        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.O) {
            return
        }
        createChannel(context, channelName)
    }

    @RequiresApi(api = Build.VERSION_CODES.O)
    private fun createChannel(context: Context, channelName: String) {
        val channel =
            NotificationChannel(channelName, channelName, NotificationManager.IMPORTANCE_MIN)
        channel.description = channelName
        channel.enableLights(false)
        channel.enableVibration(false)
        channel.vibrationPattern = longArrayOf(0)
        channel.lightColor = Color.RED
        channel.setSound(null, null)
        channel.lockscreenVisibility = Notification.VISIBILITY_PUBLIC
        channel.importance = NotificationManager.IMPORTANCE_MIN

        val notificationManager =
            context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        notificationManager.createNotificationChannel(channel)
    }
}
