//
//  EventEmitter.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Foundation
import Flutter
import JedAIKit

class EventEmitter: NSObject, FlutterStreamHandler {
    
    static let shared = EventEmitter()
    
    private static let eventsChannelName = "com.anagog.flutter/method_channel/events"
    private static let queueLabel = "com.anagog.flutter.pending.events.queue"
    
    private var pendingEventsQueue = DispatchQueue(label: queueLabel, qos: .utility)
    private var pendingEvents: [[String: Any]] = []

    private var channelEvents: FlutterEventChannel? = nil
    private var eventSink: FlutterEventSink? = nil

    static func register(with registrar: FlutterPluginRegistrar) {
        guard shared.channelEvents == nil else {
            return
        }
        shared.channelEvents = FlutterEventChannel(name: eventsChannelName, binaryMessenger: registrar.messenger())
        shared.channelEvents?.setStreamHandler(shared)
    }

    func sendEvent(_ notificationInfo: [String: Any]) {
        if let eventSink = self.eventSink {
            os_log_file("JEMAEmitter send event")
            eventSink(notificationInfo)
        }
        else {
            pendingEventsQueue.sync {
                pendingEvents.append(notificationInfo)
            }
            os_log_file("JEMAEmitter not activated yet. Event was added to the queue")
        }
    }
    
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        os_log_file("JEMAEmitter start listen")
        self.eventSink = events
        self.pendingEventsQueue.sync {
            os_log_file("JEMAEmitter there are %d pending event(s)", self.pendingEvents.count)
            while self.pendingEvents.count > 0 {
                let event = self.pendingEvents.removeFirst()
                self.sendEvent(event)
            }
        }
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        os_log_file("JEMAEmitter stop listen")
        return nil
    }
    
}
