//
//  JemaAppDelegate.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Flutter
import Foundation
import JedAIKit
import UserNotifications

extension SwiftAnagogFlutterPlugin: FlutterApplicationLifeCycleDelegate {
    
    private static var backgroundModeInitialized = false
    private static var foregroundModeInitialized = false
    
    public func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [AnyHashable : Any] = [:]) -> Bool {
        print("PLUGIN", "func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [AnyHashable : Any] = [:]) -> Bool")
        os_log_file("PLUGIN %@", #function)
        if launchOptions[UIApplication.LaunchOptionsKey.location] != nil {
            initBackgroundModeIfNeed()
        }
        else {
            initForegroundModeIfNeed()
        }
        return true
    }
    
    public func applicationWillEnterForeground(_ application: UIApplication) {
        print("PLUGIN", "func applicationWillEnterForeground(_ application: UIApplication)")
        os_log_file("PLUGIN %@", #function)
        initForegroundModeIfNeed()
    }
    
    public func applicationWillTerminate(_ application: UIApplication) {
        print("PLUGIN", "func applicationWillTerminate(_ application: UIApplication)")
        os_log_file("PLUGIN %@", #function)
        JemaWrapper.shared.shutdown()
    }
    
    public func application(_ application: UIApplication, performFetchWithCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) -> Bool {
        print("PLUGIN", "func application(_ application: UIApplication, performFetchWithCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) -> Bool")
        os_log_file("PLUGIN %@", #function)
        JemaWrapper.shared.refresh()
        completionHandler(.newData)
        return true
    }
    
    public func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        print("PLUGIN", "func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data)")
        os_log_file("PLUGIN %@", #function)
        JemaWrapper.shared.updateDeviceToken(deviceToken)
    }
    
    public func application(_ application: UIApplication, didFailToRegisterForRemoteNotificationsWithError error: Error) {
        print("PLUGIN", "func application(_ application: UIApplication, didFailToRegisterForRemoteNotificationsWithError error: Error)", error)
        os_log_file("PLUGIN %@ %@", #function, error.localizedDescription)
    }
    
    public func application(_ application: UIApplication, didReceiveRemoteNotification userInfo: [AnyHashable : Any], fetchCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) -> Bool {
        print("PLUGIN", "func application(_ application: UIApplication, didReceiveRemoteNotification userInfo: [AnyHashable : Any], fetchCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) -> Bool", userInfo)
        os_log_file("PLUGIN %@ %@", #function, userInfo)
        JemaWrapper.shared.didReceiveRemoteNotification(userInfo)
        completionHandler(.newData)
        return true
    }
    
    private func initBackgroundModeIfNeed() {
        guard Self.backgroundModeInitialized == false else {
            return
        }
        JemaWrapper.shared.startup()
        JemaWrapper.shared.delegate?.startBackgroundServices()
        Self.backgroundModeInitialized = true
    }

    private func initForegroundModeIfNeed() {
        guard Self.foregroundModeInitialized == false else {
            return
        }
        JemaWrapper.shared.startup()
        JemaWrapper.shared.delegate?.startForegroundServices()
        Self.foregroundModeInitialized = true
    }
    
}

extension SwiftAnagogFlutterPlugin: UNUserNotificationCenterDelegate {
    
    @available(iOS 12.0, *)
    public func userNotificationCenter(_ center: UNUserNotificationCenter, openSettingsFor notification: UNNotification?) {
        print("PLUGIN", "func userNotificationCenter(_ center: UNUserNotificationCenter, openSettingsFor notification: UNNotification?)")
        os_log_file("PLUGIN %@", #function)
    }
    
    public func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        // When user clicks on JEMA notification call this method to report the click event
        print("PLUGIN", "func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void)")
        os_log_file("PLUGIN %@", #function)
        JemaWrapper.shared.didReceiveNotification(response.notification)
        completionHandler()
    }

    public func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        print("PLUGIN", "func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void)")
        os_log_file("PLUGIN %@", #function)
        completionHandler([.alert, .sound])
    }
    
}
