//
//  SwiftAnagogFlutterPlugin.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Flutter
import Foundation
import JedAIKit
import JedAIJEMAKit
import JedAIMetricsKit
import JedAIReportKit
import CoreMotion
import CoreLocation

public class SwiftAnagogFlutterPlugin: NSObject, FlutterPlugin {

    private static let methodsChannelName = "com.anagog.flutter/method_channel"
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = SwiftAnagogFlutterPlugin()
        let channel = FlutterMethodChannel(name: methodsChannelName, binaryMessenger: registrar.messenger())

        registrar.addApplicationDelegate(instance)
        registrar.addMethodCallDelegate(instance, channel: channel)
        EventEmitter.register(with: registrar)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "syncCampaigns":
            JemaAPI.shared.downloadCampaigns()
            result(nil)
        case "simulateCampaign":
            JemaAPI.shared.simulateCampaign((call.arguments as? [String: String])?["campaign_identifier"])
            result(nil)
        case "getUserDefinedString":
            guard let key = call.arguments as? String else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            result(JemaAPI.shared.getUserDefinedString(key))
        case "getUserDefinedDecimal":
            guard let key = call.arguments as? String else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            result(JemaAPI.shared.getUserDefinedDecimal(key))
        case "getUserDefinedInteger":
            guard let key = call.arguments as? String else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            result(JemaAPI.shared.getUserDefinedInteger(key))
        case "setUserDefinedString":
            guard let arguments = call.arguments as? [String: Any],
                  let key = arguments["key"] as? String,
                  let value = arguments["value"] as? String
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            JemaAPI.shared.setUserDefinedString(key, value: value)
            result(nil)
        case "setUserDefinedDecimal":
            guard let arguments = call.arguments as? [String: Any],
                  let key = arguments["key"] as? String,
                  let value = arguments["value"] as? Double
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            JemaAPI.shared.setUserDefinedDecimal(key, value: value)
            result(nil)
        case "setUserDefinedInteger":
            guard let arguments = call.arguments as? [String: Any],
                  let key = arguments["key"] as? String,
                  let value = arguments["value"] as? Int
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            JemaAPI.shared.setUserDefinedInteger(key, value: value)
            result(nil)
        case "getVersion":
            result(JemaAPI.shared.getVersion())
        case "getUserDefinedStat":
            result(JemaAPI.shared.getUserDefinedMetrics())
        case "fireMicromoment":
            guard let micromoment = call.arguments as? [String: Any] else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            JemaAPI.shared.fireMicromoment(micromoment)
            result(nil)
        case "share":
            JemaAPI.shared.share {
                result(nil)
            }
        case "forceScheduleReports":
            JemaAPI.shared.forceScheduleReports()
            result(nil)
        case "requestMicrosegmentsSnapshotReport":
            JemaAPI.shared.requestMicrosegmentsSnapshotReport()
            result(nil)
        case "sendUserInteractionEvent":
            guard let arguments = call.arguments as? [String: Any],
                  let campaignIdentifier = arguments["campaign_identifier"] as? String,
                  let triggerType = arguments["trigger_type"] as? String
            else {
                result(FlutterError.wrongArguments(call.arguments))
                return
            }
            if JemaAPI.shared.sendUserInteractionEvent(campaignIdentifier: campaignIdentifier, triggerType: triggerType) == true {
                result(nil)
            }
            else {
                result(FlutterError.wrongArguments(call.arguments))
            }
        case "isJedAIReady":
            result(JemaAPI.shared.isJedAIReady())
        case "getCampaigns":
            result(JemaAPI.shared.getCampaigns())
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
}

extension FlutterError {
    static func wrongArguments(_ arguments: Any?) -> FlutterError {
        return FlutterError(code: "", message: "Unable to corectly parse input arguments", details: arguments)
    }
}
