library anagog_flutter;

import 'package:anagog_flutter_platform_interface/anagog_flutter_platform_interface.dart';

export 'package:anagog_flutter_platform_interface/src/model/campaign.dart';
export 'package:anagog_flutter_platform_interface/src/model/micromoment.dart';
export 'package:anagog_flutter_platform_interface/src/model/location.dart';
export 'package:anagog_flutter_platform_interface/src/model/constants.dart';
export 'package:anagog_flutter_platform_interface/src/model/event_type.dart';

class AnagogFlutter extends AnagogFlutterPlatform {
  static AnagogFlutter? _instance;

  AnagogFlutter._();

  static AnagogFlutter get instance => _instance ??= AnagogFlutter._();

  ///Simulate campaign occuranc, use this API to test your camaigns as defined.
  ///[campaignIdentifier] A campaign identifier to simulate. If null simulates all campaigns
  @override
  Future simulateCampaign(String? campaignIdentifier) async =>
      AnagogFlutterPlatform.instance.simulateCampaign(campaignIdentifier);

  ///Sync campaigns
  ///Download all your campaigns to your app
  @override
  Future syncCampaigns() async =>
      AnagogFlutterPlatform.instance.syncCampaigns();

  ///Create an EventStream from SDK to your Flutter app.
  ///Use this API to listen for SDK events such as onCampaignTriggered, onNotificationClick
  @override
  Stream obtainEventStream() =>
      AnagogFlutterPlatform.instance.obtainEventStream();

  ///Get the app custom micro segments map
  @override
  Future<Map<dynamic, dynamic>?> getUserDefinedStat() async =>
      AnagogFlutterPlatform.instance.getUserDefinedStat();

  ///Set String custom micro segment value
  ///[arguments] Micro Segment values
  @override
  Future<void> setUserDefinedString(Map<String, dynamic> arguments) async =>
      AnagogFlutterPlatform.instance.setUserDefinedString(arguments);

  ///Get String custom micro segment
  ///[name] Stat name
  @override
  Future<String?> getUserDefinedString(String name) async =>
      AnagogFlutterPlatform.instance.getUserDefinedString(name);

  ///Set Integer custom micro segment value
  ///[arguments] Micro Segment values
  @override
  Future<void> setUserDefinedInteger(Map<String, dynamic> arguments) async =>
      AnagogFlutterPlatform.instance.setUserDefinedInteger(arguments);

  ///Get Integer custom micro segment
  ///[name] Stat name
  @override
  Future<int?> getUserDefinedInteger(String name) async =>
      AnagogFlutterPlatform.instance.getUserDefinedInteger(name);

  ///Set Decimal custom micro segment
  ///[arguments] Micro Segment values
  @override
  Future<void> setUserDefinedDecimal(Map<String, dynamic> arguments) async =>
      AnagogFlutterPlatform.instance.setUserDefinedDecimal(arguments);

  ///Get Decimal custom micro segment
  ///[name] Stat name
  @override
  Future<double?> getUserDefinedDecimal(String name) async =>
      AnagogFlutterPlatform.instance.getUserDefinedDecimal(name);

  ///Use to create report a [Micromoment] object from your app
  @override
  Future fireMicromoments(Micromoment micromoment) async =>
      AnagogFlutterPlatform.instance.fireMicromoments(micromoment);

  ///Create and share SDK report.
  ///Use this in case you want to share with Anagog SDK operation and debugging report for analyze
  @override
  Future share() async => AnagogFlutterPlatform.instance.share();

  ///Get SDK Version
  @override
  Future getVersion() async => AnagogFlutterPlatform.instance.getVersion();

  ///Request to generate Microsegments Snapshot report.
  @override
  Future requestMicrosegmentsSnapshotReport() async =>
      AnagogFlutterPlatform.instance.requestMicrosegmentsSnapshotReport();

  ///Generate and send SDK reports
  @override
  Future forceScheduleReports() async =>
      AnagogFlutterPlatform.instance.forceScheduleReports();

  ///Forward user interaction event to SDK
  @override
  Future setUserInteractionEvent(
          String campaignIdentifier, AnagogActivityType activityType) async =>
      AnagogFlutterPlatform.instance
          .setUserInteractionEvent(campaignIdentifier, activityType);

  @override
  Future<List<Campaign>?> getCampaigns() async =>
      AnagogFlutterPlatform.instance.getCampaigns();

  @override
  Future getPluginVersion() async {
    return "1.2.0";
  }

  @override
  Future isJedAIReady() async => AnagogFlutterPlatform.instance.isJedAIReady();
}
